using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using WebControls.PartsWebService;
using System.Data;
using Common.Data;
using System.Drawing;
using GeneralUtility;

namespace TNT.WebControlsLib 
{
	[DefaultProperty("Text"),
	ToolboxData("<{0}:NewPart runat=server></{0}:NewPart>")]
	public class NewPart : WebControl,INamingContainer
	{
		
		protected Label   originLbl                    = new Label();
		protected Label   destLbl                      = new Label();
		protected TextBox partNumberTxtBox             = new TextBox();
		protected TextBox descriptionTxtBox            = new TextBox();
		protected Button  addBtn                       = new Button();
		protected int    customerId;
		protected int    originId;
		protected int    destId;
		protected string originAlias;
		protected string destAlias;
		protected string originName;
		protected string destName;
		public event EventHandler PartAdded;//public event
		protected const int PART_NUMBER_MAX_CHARS      = 30;//varchar(30)
		protected const int PART_DESCRIPTION_MAX_CHARS = 30;
		protected const string ORIGIN                  = "Origin: ";
		protected const string DESTINATION             = "Destination: ";
		protected const string PART_NUMBER             = "Part Number: ";
		protected const string DESCRIPTION             = "Description: ";
		protected const string ADD                     = "Add";
		//
		public NewPart()
		{
			originLbl.CssClass=AppConstants.TABLE_CELL_CSS_CLASS ;
			destLbl.CssClass  =AppConstants.TABLE_CELL_CSS_CLASS;
		}
		// method to invoke the event handler
		protected void OnPartAdded(NewPartEventArgs e)
		{
			if(PartAdded != null)
			{
				PartAdded(this,e);// call handlers
			}
		}
		private void initCtrlProps()
		{
			addBtn.CssClass=AppConstants.BUTTON_CSS_CLASS;
			addBtn.CausesValidation=false;
			// add handler for submit button
			addBtn.Click += new EventHandler(this.addPart);
			addBtn.Text=ADD;
			partNumberTxtBox.MaxLength =PART_NUMBER_MAX_CHARS;
			descriptionTxtBox.MaxLength=PART_DESCRIPTION_MAX_CHARS;
		}
		protected override void CreateChildControls()
		{
			initCtrlProps();
			
			Label olbl = new Label();olbl.CssClass=AppConstants.LABEL_CSS_CLASS;
			Label dlbl = new Label();dlbl.CssClass=AppConstants.LABEL_CSS_CLASS;
			Label pnlbl= new Label();pnlbl.CssClass=AppConstants.LABEL_CSS_CLASS;
			Label desclbl = new Label();desclbl.CssClass=AppConstants.LABEL_CSS_CLASS;
			olbl.Text=ORIGIN;dlbl.Text=DESTINATION;pnlbl.Text=PART_NUMBER;desclbl.Text=DESCRIPTION;
			
			Controls.Add(new LiteralControl("<fieldset>"));
			Controls.Add(new LiteralControl("<legend class='legend'>"));
			Controls.Add(new LiteralControl("New Part: </legend>"));
			// create the table and set table properties
			Table table  = new Table()   ;
			table.CellSpacing= AppConstants.TABLE_CELL_SPACING  ;
			table.CellPadding= AppConstants.TABLE_CELL_PADDING;
			// create first row
			TableRow row   = new TableRow();
			// create cells for first row
			TableCell cell = new TableHeaderCell();
			cell.Attributes.Add("align","left");
			cell.Controls.Add(olbl);
			row.Cells.Add(cell);// add first cell
			cell = new TableCell();
			cell.Controls.Add(originLbl);//add second cell
			row.Cells.Add(cell);
			// add first row to table
			table.Rows.Add(row);//add first row
			// create second row
			row = new TableRow();
			cell = new TableHeaderCell();
			cell.Attributes.Add("align","left");
			cell.Controls.Add(dlbl);
			row.Cells.Add(cell);//add first cell to second row
			cell = new TableCell();
			cell.Controls.Add(destLbl);
			row.Cells.Add(cell);//add second cell to second row
			// add second row to table
			table.Rows.Add(row);//add 2nd row
			// create third row
			row = new TableRow();
			cell = new TableHeaderCell();
			cell.Attributes.Add("align","left");
			cell.Controls.Add(pnlbl);
			row.Cells.Add(cell);//add first cell to 3rd row
			cell = new TableCell();
			cell.Controls.Add(partNumberTxtBox);
			
			row.Cells.Add(cell);
			// add 3rd row to table
			table.Rows.Add(row);//add 3rd row
			row = new TableRow();
			cell = new TableHeaderCell();
			cell.Attributes.Add("align","left");
			cell.Controls.Add(desclbl);
			row.Cells.Add(cell);
			cell = new TableCell();
			cell.Controls.Add(descriptionTxtBox);
			row.Cells.Add(cell);
			table.Rows.Add(row);//add 4th row
			// create last row
			row = new TableRow();
			cell = new TableCell();
			cell.Controls.Add(addBtn);
			row.Cells.Add(cell);
			row.Cells.Add(new TableCell());
			table.Rows.Add(row);//add 5th row
			// add table to this
			Controls.Add(table);
			Controls.Add(new LiteralControl("</fieldset>"));
		}
		private bool areCtrlsValid()
		{
			return (partNumberTxtBox.Text.Trim().Length != 0);
		}
		protected void addPart(object sender,EventArgs e)
		{
			if(areCtrlsValid())
			{
				string part = partNumberTxtBox.Text.Trim();
				try
				{
					int newPartPackagingId = new PartsDataService().insertNewPart(customerId,originId,destId,
						part,descriptionTxtBox.Text.Trim());
					string result="Unable to add "+part+".";
					if(newPartPackagingId == 0)//part already exists
					{
						result = "Part "+part+" already on file.";
					}
					else if(newPartPackagingId == -1)// exception while adding part
					{
						result = "Unable to insert part"+part+".";
					}
					else
					{
						result = "Successfully inserted part "+part+".";
					}
					OnPartAdded(new NewPartEventArgs(newPartPackagingId,result));//trigger the part added event
				}
				catch(Exception eex)
				{
					OnPartAdded(new NewPartEventArgs(-1,eex.Message));
				}
				
			}
			else
			{
				OnPartAdded(new NewPartEventArgs(-1,"Unable to insert part: Missing Part Number."));
			}
		}
		public int OriginId
		{
			get
			{
				return originId;
			}
			set
			{
				originId=value;
			}
		}
		public int DestinationId
		{
			get
			{
				return destId;
			}
			set
			{
				destId=value;
			}
		}
		public string OriginAlias
		{
			get
			{
				return originAlias;
			}
			set
			{
				originAlias=value;
				originLbl.Text=originAlias+","+originName;
			}
		}
		public string OriginName
		{
			get
			{
				return originName;
			}
			set
			{
				originName=value;
				originLbl.Text=originAlias+","+originName;
			}
		}
		public string DestAlias
		{
			get
			{
				return destAlias;
			}
			set
			{
				destAlias=value;
				destLbl.Text=destAlias+","+destName;
			}
		}
		public string DestName
		{
			get
			{
				return destName;
			}
			set
			{
				destName=value;
				destLbl.Text=destAlias+","+destName;
			}
		}
		public int CustomerId
		{
			get
			{
				return customerId;
			}
			set
			{
				customerId=value;
			}
		}
		
	}
}

