using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using WebControls.PartsWebService;
using System.Data;
using Common.Data;
using GeneralUtility;

namespace TNT.WebControlsLib
{
	/// <summary>
	/// Summary description for WebCustomControl1.
	/// </summary>
	[DefaultProperty("Text"),
	ToolboxData("<{0}:PartsPackagingBrief runat=server></{0}:PartsPackagingBrief>")]
	public class PartsPackagingBrief : System.Web.UI.WebControls.WebControl
	{
		protected const string CUBE_FORMAT="{0:f2}";
		protected const string WEIGHT_FORMAT="{0:f3}";	
		public class PackagingTable :Table
		{
			protected const string EMPTY_CELL="&nbsp;";
			protected int packagingId;
			protected DisplayUnits units ;
			public PackagingTable(int packId,DisplayUnits u)
			{
				packagingId=packId;
				units = u;
				buildHdr();
				buildCells();
				buildTlr();
			}
			protected TableCell getEmptyCell()
			{
				TableCell tc = new TableCell();tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.Text=EMPTY_CELL;tc.BorderStyle=BorderStyle.None;
				return tc;
			}
			protected TableHeaderCell getHdrCell(string title)
			{
				TableHeaderCell hdr = new TableHeaderCell();
				hdr.BorderStyle=BorderStyle.None;hdr.CssClass   = AppConstants.LABEL_CSS_CLASS;
				hdr.Text=title;
				return hdr;
			}
			protected void buildHdr()
			{
				TableRow tr   = new TableRow();
				TableCell tc  = new TableHeaderCell();
				tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.Text="Part Number: ";tc.BorderStyle=BorderStyle.None;
				tr.Cells.Add(tc);
				tc = getEmptyCell();tc.ColumnSpan=3;
				tr.Cells.Add(tc);
				Rows.Add(tr);
				tr   = new TableRow();
				tc  = new TableHeaderCell();tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.BorderStyle=BorderStyle.None;tc.Text="Description: ";
				tr.Cells.Add(tc);
				tc = getEmptyCell();tc.ColumnSpan=3;
				tr.Cells.Add(tc);
				Rows.Add(tr);
				//
				tr   = new TableRow();
				tc  = new TableHeaderCell();tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.BorderStyle=BorderStyle.None;tc.Text="Origin: ";
				tr.Cells.Add(tc);
				tc = getEmptyCell();tc.ColumnSpan=3;
				tr.Cells.Add(tc);
				Rows.Add(tr);
				//
				tr   = new TableRow();
				tc  = new TableHeaderCell();tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.BorderStyle=BorderStyle.None;tc.Text="Destination: ";
				tr.Cells.Add(tc);
				tc = getEmptyCell();tc.ColumnSpan=3;
				tr.Cells.Add(tc);
				Rows.Add(tr);
				//
				tr = new TableRow();
				tr.Cells.Add(getEmptyCell());
				tr.Cells.Add(getHdrCell("Primary"));
				tr.Cells.Add(getHdrCell("Secondary"));
				tr.Cells.Add(getEmptyCell());
				Rows.Add(tr);
			}
			protected void addBlankRow(string rowHdr,string rowTlr)
			{
				TableRow tr = new TableRow();
				TableCell tc = new TableHeaderCell();
				tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.Text=rowHdr;tc.BorderStyle=BorderStyle.Solid;
				tr.Cells.Add(tc);
				tc = new TableCell();tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.Text=EMPTY_CELL; tc.BorderStyle=BorderStyle.Solid;
				tr.Cells.Add(tc);
				tc = new TableCell();tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.Text=EMPTY_CELL;tc.BorderStyle=BorderStyle.Solid;
				tr.Cells.Add(tc);
				tc = new TableCell();tc.CssClass   = AppConstants.LABEL_CSS_CLASS;
				tc.Text=rowTlr;tc.BorderStyle=BorderStyle.None;
				tr.Cells.Add(tc);
				this.Rows.Add(tr);
			}
			protected void buildCells()
			{
				string length="in",volume="ft<sup>3</sup>",weight="lb";
				if(units==DisplayUnits.METRIC)
				{
					length="cm";
					volume="m<sup>3</sup>";
					weight="kg";
				}
				
				addBlankRow("Class: ",EMPTY_CELL);
				addBlankRow("Container: ",EMPTY_CELL);
				addBlankRow("Quantity: ",EMPTY_CELL);
				addBlankRow("Length: ",length);
				addBlankRow("Width: ",length);
				addBlankRow("Height: ",length);
				addBlankRow("Cube: ",volume);
				addBlankRow("Weight: ",weight);
				addBlankRow("Stack Qty: ",EMPTY_CELL);
				addBlankRow("Rounding: ",EMPTY_CELL);
			}
			protected void buildTlr()
			{
			}
			public void addRowAt(int rowIndex,string priValue,string secValue)
			{
				Rows[rowIndex].Cells[1].Text=priValue;
				Rows[rowIndex].Cells[2].Text=secValue;
			}
		}
		protected string text;
		protected int packagingId;
		public int PackagingId
		{
			get
			{
				return packagingId;
			}
			set
			{
				packagingId=value;
			}
		}
		public enum DisplayUnits
		{
			METRIC,ENGLISH
		}
		protected DisplayUnits unit = DisplayUnits.ENGLISH;
		protected enum PartsPackagingTableCols
		{
			PartNumber,Description,PriContainerId,PriQty,PriLength,PriWidth,PriHeight,PriCube,PriWeight,
			PriStackable,PriStackQty,Rounding,RoundingQty,SecContainerId,SecQty,SecLength,SecWidth,
			SecHeight,SecCube,SecWeight,SecStackable,SecStackQty,NeedsAttention,PriClass,SecClass,OriginName,DestinationName
		}
		public DisplayUnits Unit
		{
			get
			{
				return unit;
			}
			set
			{
				unit=value;
			}
		}
		[Bindable(true),Category("Appearance"),DefaultValue("")]
		public string Text
		{
			get
			{
				return text;
			}

			set
			{
				text = value;
			}
		}
		protected string getLength(string lengthInEnglish)
		{
			string l=lengthInEnglish;
			if(unit==DisplayUnits.METRIC)
			{
				try
				{
					double d = Double.Parse(lengthInEnglish);
					double lengthInCm = GeneralUtility.UnitConverter.inchesToCentimeters(d);
					l = lengthInCm.ToString();
					
				}
				catch(Exception)
				{
				}
			}
			return l;
		}
		protected string getCube(string cubeInEnglish)
		{
			string c = cubeInEnglish;
			if(unit==DisplayUnits.METRIC)
			{
				try
				{

					double d = Double.Parse(cubeInEnglish);
					double cubeInMeters = GeneralUtility.UnitConverter.cubicFeetToCubicMeters(d);
					// get a culture that used the "." as the decimal separator
					System.Globalization.CultureInfo newInfo = new System.Globalization.CultureInfo(0x0C09);
					c = string.Format(newInfo.NumberFormat,CUBE_FORMAT,cubeInMeters);
				}
				catch(Exception){}
			}
			return c;
		}
		protected string getWeight(string weightInEnglish)
		{
			string w = weightInEnglish;
			if(unit==DisplayUnits.METRIC)
			{
				try
				{
					double d = Double.Parse(weightInEnglish);
					double weightInKgs = GeneralUtility.UnitConverter.poundsToKilograms(d);
					// get a culture that used the "." as the decimal separator
					System.Globalization.CultureInfo newInfo = new System.Globalization.CultureInfo(0x0C09);
					w = string.Format(newInfo.NumberFormat,WEIGHT_FORMAT,weightInKgs);
				}
				catch(Exception){}
			}
			return w;
		}
		protected override void CreateChildControls()
		{
			DataSet ds = new PartsDataService().getPartPackagingTable(packagingId);
			DataTable dt = null;
			if(ds != null)
			{
				dt = ds.Tables[PartsBriefTable.name];
				PackagingTable pt = new PackagingTable(packagingId,unit);
				if(dt != null && dt.Rows != null && dt.Rows.Count == 1)
				{
					foreach(DataRow row in dt.Rows)
					{
						string number="&nbsp;",description="&nbsp;",priClass="&nbsp;",secClass="&nbsp;";
						string priContainerId="&nbsp;",secContainerId="&nbsp;",priQty="&nbsp;",secQty="&nbsp;";
						string priLength="&nbsp;",secLength="&nbsp;",priWidth="&nbsp;",secWidth="&nbsp;",priHeight="&nbsp;",secHeight="&nbsp;";
						string priCube="&nbsp;",secCube="&nbsp;",priWeight="&nbsp;",secWeight="&nbsp;",priStackQty="&nbsp;",secStackQty="&nbsp;";
						string originName="&nbsp;",destinationName="&nbsp;";
						//
						if(row[(int)PartsPackagingTableCols.OriginName] != System.DBNull.Value)
						{
							originName = (string)row[(int)PartsPackagingTableCols.OriginName];
						}
						if(row[(int)PartsPackagingTableCols.DestinationName] != System.DBNull.Value)
						{
							destinationName = (string)row[(int)PartsPackagingTableCols.DestinationName];
						}
						if(row[(int)PartsPackagingTableCols.PartNumber] != System.DBNull.Value)
						{
							number = (string)row[(int)PartsPackagingTableCols.PartNumber];
						}
						if(row[(int)PartsPackagingTableCols.Description] != System.DBNull.Value)
						{
							description = (string)row[(int)PartsPackagingTableCols.Description];
						}
						if(row[(int)PartsPackagingTableCols.PriClass] != System.DBNull.Value)
						{
							priClass = row[(int)PartsPackagingTableCols.PriClass].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecClass] != System.DBNull.Value)
						{
							secClass = row[(int)PartsPackagingTableCols.SecClass].ToString();
						}
						if(row[(int)PartsPackagingTableCols.PriContainerId] != System.DBNull.Value)
						{
							priContainerId = row[(int)PartsPackagingTableCols.PriContainerId].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecContainerId] != System.DBNull.Value)
						{
							secContainerId = row[(int)PartsPackagingTableCols.SecContainerId].ToString();
						}
						if(row[(int)PartsPackagingTableCols.PriQty] != System.DBNull.Value)
						{
							priQty = row[(int)PartsPackagingTableCols.PriQty].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecQty] != System.DBNull.Value)
						{
							secQty = row[(int)PartsPackagingTableCols.SecQty].ToString();
						}
						if(row[(int)PartsPackagingTableCols.PriLength] != System.DBNull.Value)
						{
							priLength = row[(int)PartsPackagingTableCols.PriLength].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecLength] != System.DBNull.Value)
						{
							secLength = row[(int)PartsPackagingTableCols.SecLength].ToString();
						}

						if(row[(int)PartsPackagingTableCols.PriWidth] != System.DBNull.Value)
						{
							priWidth = row[(int)PartsPackagingTableCols.PriWidth].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecWidth] != System.DBNull.Value)
						{
							secWidth = row[(int)PartsPackagingTableCols.SecWidth].ToString();
						}
						if(row[(int)PartsPackagingTableCols.PriHeight] != System.DBNull.Value)
						{
							priHeight = row[(int)PartsPackagingTableCols.PriHeight].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecHeight] != System.DBNull.Value)
						{
							secHeight = row[(int)PartsPackagingTableCols.SecHeight].ToString();
						}
                        if(row[(int)PartsPackagingTableCols.PriCube] != System.DBNull.Value)
						{
							priCube = row[(int)PartsPackagingTableCols.PriCube].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecCube] != System.DBNull.Value)
						{
							secCube = row[(int)PartsPackagingTableCols.SecCube].ToString();
						}
						if(row[(int)PartsPackagingTableCols.PriWeight] != System.DBNull.Value)
						{
							priWeight = row[(int)PartsPackagingTableCols.PriWeight].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecWeight] != System.DBNull.Value)
						{
							secWeight = row[(int)PartsPackagingTableCols.SecWeight].ToString();
						}
						if(row[(int)PartsPackagingTableCols.PriStackQty] != System.DBNull.Value)
						{
							priStackQty = row[(int)PartsPackagingTableCols.PriStackQty].ToString();
						}
						if(row[(int)PartsPackagingTableCols.SecStackQty] != System.DBNull.Value)
						{
							secStackQty = row[(int)PartsPackagingTableCols.SecStackQty].ToString();
						}
						//
						pt.Rows[0].Cells[1].Text=number;
						pt.Rows[1].Cells[1].Text=description;
						pt.Rows[2].Cells[1].Text=originName;
						pt.Rows[3].Cells[1].Text=destinationName;
						// 4 is primary -- seconday
						int startingIndex = 5;
						pt.addRowAt(startingIndex,priClass,secClass);
						pt.addRowAt(startingIndex+1,priContainerId,secContainerId);
						pt.addRowAt(startingIndex+2,priQty,secQty);
						pt.addRowAt(startingIndex+3,getLength(priLength),getLength(secLength));
						pt.addRowAt(startingIndex+4,getLength(priWidth),getLength(secWidth));
						pt.addRowAt(startingIndex+5,getLength(priHeight),getLength(secHeight));
						pt.addRowAt(startingIndex+6,getCube(priCube),getCube(secCube));
						pt.addRowAt(startingIndex+7,getWeight(priWeight),getWeight(secWeight));
						pt.addRowAt(startingIndex+8,priStackQty,secStackQty);
						Controls.Add(pt);
					}
				}
				else
				{
					Label msg = new Label();
					msg.CssClass="label";
					msg.Text="Packaging data is not available for packagingId = : "+packagingId;
					Controls.Add(msg);
				}
			}
			
		}
	}
}
