#include <assert.h>

#include <openssl/bn.h>
#include <openssl/pem.h>

#include "keys.h"


unsigned char fakekey[] = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                   0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                   0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                   0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08 
                 };

int keys_gen_symmetric_key(void **keybuf, 
                           size_t keylength)
{
    assert(keybuf);
    assert(keylength <= 32);

    *keybuf = malloc(keylength);
    memcpy(*keybuf, fakekey, keylength);
    return 0;
}


int keys_free_symmetric_key(void *keybuf)
{
    assert(keybuf);

    free(keybuf);
    return 0;
}


int keys_gen_rsa_key(RSA **rsa_key,
                     size_t size)
{
   assert(rsa_key);

   *rsa_key = RSA_generate_key (size, 3, NULL, NULL);
   assert(*rsa_key);
   return 0;
}


int keys_set_rsa_pubkey_by_mod_e(RSA **rsa_key,
                                 const char *mod, 
                                 const char *exp)
{
   assert(rsa_key);
   
   *rsa_key = RSA_new();
   assert(*rsa_key);

   BN_hex2bn(&((*rsa_key)->n), mod);
   BN_hex2bn(&((*rsa_key)->e), exp);
   (*rsa_key)->d = NULL;
   (*rsa_key)->p = NULL;
   (*rsa_key)->q = NULL;
   (*rsa_key)->dmp1 = NULL;
   (*rsa_key)->dmq1 = NULL;
   (*rsa_key)->iqmp = NULL;

   return 0;
}


int keys_free_rsa_key(RSA *rsa_key)
{
    assert(rsa_key);

    RSA_free(rsa_key);
    return 0;
}


int keys_rsakey2file(const char *filename,
                     KEYS_KEYTYPE key_type, 
                     RSA *rsa_key)
{
    BIO *bio;

    assert(filename);
    assert(rsa_key);

    bio = BIO_new_file(filename, "w+");
    if (key_type == KEYS_PRIVKEY) {
        PEM_write_bio_RSAPrivateKey(bio, rsa_key,
                                    NULL, NULL, 0, NULL, NULL);
    }
    else {
        PEM_write_bio_RSAPublicKey(bio, rsa_key);
    }

    BIO_free(bio);

    return 0;    
}


int keys_file2rsakey(RSA **rsa_key,
                     const char *filename,
                     KEYS_KEYTYPE key_type)
{
    BIO *bio;

    assert(filename);
    assert(rsa_key);

    bio = BIO_new_file(filename, "r");
    if (key_type == KEYS_PRIVKEY) {
        PEM_read_bio_RSAPrivateKey(bio, rsa_key,
                                   NULL, NULL);
    }
    else {
        PEM_read_bio_RSAPublicKey(bio, rsa_key,
                                  NULL, NULL);
    }

    BIO_free(bio);

    return 0;
}
