// file: base.cpp
// Implementation of the Base class.
// Copyright 2001, 2003 Jim Hyslop.
//
// This file may be freely used, modified and distributed, provided that
// the accompanying copyright notice remains intact.
//
// $Id: base.cpp,v 1.14 2003/03/09 16:22:27 Jim Exp $

#include "base.h"
#include <iostream>
#include <sstream>
#include "factory.h"

using namespace std;

// The unique identifier for the class.  Now, you may be wondering
// why not use rtti, as in: 'typeid(*this).name();'  The reason is,
// typeid().name() returns a compiler-specific string.  That would
// tie this class, and its streamed type, to a specific compiler,
// or potentially worse a specific version of a specific compiler.
// By defining our own type, we are now platform-independent.  The
// down-side, of course, is that now we have to make sure each type
// has a unique identifier - hence I'm using the class name.
Base::BaseKeyType Base::classID() const
{
    return baseClassID();
}

Base::BaseKeyType Base::baseClassID()
{
    return "Base";
}

Base::Base( int value ) 
: value_( value ) 
{
}

Base::Base( const BaseParams & params )
: value_ ( params.value_ )
{
}


auto_ptr< Base::BaseParams > Base::createStorageInfo( ) const
{
    auto_ptr< BaseParams > ret( new BaseParams );
    return ret;
}

void Base::setStorageValues( BaseParams & bp ) const
{
    bp.value_ = value_;
}

void Base::displayValues()
{
    std::cout << "Hi, I'm a Base class and my value is " << value_ << "." << std::endl;
}

Base::~Base()
{
}

