// base.h
// Copyright 2001, 2003 Jim Hyslop.
//
// This file may be freely used, modified and distributed, provided that
// the accompanying copyright notice remains intact.
//
// $Id: base.h,v 1.17 2003/04/02 03:41:48 Jim Exp $
//
// Declaration of the base class to be stored in the factory.

#ifndef BASE_HEADER_DEFINED
#define BASE_HEADER_DEFINED
#include <iosfwd>
#include <string>
#include "factory.h"

// The class to be stored in the factory template.
class Base
{
public:
    typedef std::string BaseKeyType;

    // Normal 'tors
    Base( int value );
    virtual ~Base();

    // The class's main business logic:
    virtual void displayValues();

    ////////////////////////////////////////////////////////////////////
    // persistence
    template<
        typename Source, 
        typename Sink,
        typename RestoreFn,
        typename SaveFn
        >
    void write( Sink & o ) const
    {
        std::cout << "Saving object" << std::endl;
        PersistentStorage< std::string, Sink >( o ).save( classID() );
        std::auto_ptr< BaseParams > params = createStorageInfo();
        setStorageValues( *params );
        GenericFactory
        <
            Base,
            BaseKeyType,
            Source,
            Sink,
            BaseParams,
            RestoreFn,
            SaveFn
        >::instance().save( classID(), o, *params );
    }


    struct BaseParams
    {
        int value_;
    };

    explicit Base( const BaseParams & );

    static BaseKeyType baseClassID();

protected:
    virtual std::auto_ptr< BaseParams > createStorageInfo() const;
    virtual void setStorageValues( BaseParams & ) const;

private:
    // Derived classes must override this function, and
    // provide their own classID value.
    virtual BaseKeyType classID() const;

    // The class's data to be stored and retrieved.
    int value_;

};

template< typename Storage >
class PersistentStorage< Base::BaseParams, Storage >
{
public:
    explicit PersistentStorage( Storage & os ) : warehouse_( os )
    {
    }
    Base::BaseParams & restore()
    {
        std::cout << "Restoring BaseParams" << std::endl;
        param_.value_ = PersistentStorage< int, Storage>( warehouse_ ).restore();
        return param_;
    }
    void save( const Base::BaseParams & bp ) const
    {
        PersistentStorage< int, Storage >( warehouse_ ).save( bp.value_ );
    }
protected:
    Storage & warehouse_;
    Base::BaseParams param_;
};

#endif // sentry
