// base_stream.cpp
// Example and test file to illustrate how to use the factory template.
// This file represents the stream-based I/O for saving & restoring
// objects derived from Base.
// Copyright 2001, 2003 Jim Hyslop.
//
// This file may be freely used, modified and distributed, provided that
// the accompanying copyright notice remains intact.
//
// $Id: base_stream.cpp,v 1.9 2003/04/02 03:32:29 Jim Exp $

#include "base_stream.h"
#include "derived.h"
#include <iostream>
#include <string>
#include <fstream>
#include "persistent_storage_stream_specializations.h"

using namespace std;
using namespace boost;

// Register the creation function with the factory.
namespace
{
    // Note that this highlights one of the dangers of this technique.  We have
    // no control over the order in which various modules will call registerFunctions,
    // so this technique cannot be safely used if the factory is required
    // to instantiate an object before main() executes.
    RegisterInFactoryWithParams
    < 
        Base, 
        Base::BaseParams, 
        Base, 
        Base::BaseParams, 
        istream, 
        ostream
    > registerBase( Base::baseClassID() );
}

namespace {
    
    RegisterInFactoryWithParams
    <
        Base, 
        Base::BaseParams,
        Derived, 
        Derived::DerivedParams, 
        istream, 
        ostream
    > registerWithParams( Derived::derivedClassID() );
}

void saveOutputFile(ostream & outfile, 
                    vector< shared_ptr< Base > > &loadedClasses)
{
    outfile << loadedClasses.size() << " objects in the list" << endl;
    int count;
    
    for (count=0; count < loadedClasses.size(); count++) {
        loadedClasses[count]->write< 
            istream, 
            ostream,
            BaseStreamRestoreFn,
            BaseStreamSaveFn
            >( outfile );
    }
}

void loadInputFile(istream & infile,
                   vector< shared_ptr<Base> > &loadedClasses)
{
    int count;

    loadedClasses.clear();
    int numClasses;

    // The first line in the file contains the number of classes.
    infile >> numClasses;
    loadedClasses.resize(numClasses);

    // Ignore the rest of the line
    string tmp;
    getline(infile, tmp);
    BaseFactoryUsingStreams & factory = BaseFactoryUsingStreams::instance();

    // Reload the array now.
    for (count=0; count < loadedClasses.size(); count++) {
        // Step 1: Retrieve the identifier from the file.
        string className = PersistentStorage< string, istream >( infile ).restore();
        cout << "Loading class " << className << endl;

        // Step 2: Pass that identifier to the factory, to load that
        //         object from the stream.
        auto_ptr<Base> newBase( factory.create( className, infile ) );
        if ( newBase.get() == NULL )
        {
            cout << "Error loading class " << count+1 << " (" << className << ")" << endl;
        }
        loadedClasses[count] = newBase;
    }
    cout << "Loaded " << loadedClasses.size() << " objects" << endl;
}

