// file: derived.h
// Copyright 2001, 2003 Jim Hyslop.
//
// This file may be freely used, modified and distributed, provided that
// the accompanying copyright notice remains intact.
//
// $Id: derived.h,v 1.10 2003/04/02 03:44:32 Jim Exp $
#ifndef DERIVED_CLASS_DEFINED
#define DERIVED_CLASS_DEFINED

#include "base.h"
#include <string>

class Derived : public Base
{
public:
    virtual void displayValues();

    struct DerivedParams : public Base::BaseParams
    {
        std::string word_;
        DerivedParams() {}
        DerivedParams & operator=( const BaseParams & bp )
        {
            BaseParams::operator=(bp);
            return *this;
        }
    };

    Derived( int baseValue, const std::string & word );
    explicit Derived( const DerivedParams & );
    static BaseKeyType derivedClassID();

protected:

    virtual std::auto_ptr< BaseParams > createStorageInfo() const;
    virtual void setStorageValues( BaseParams & ) const;

private:
    std::string word_;
    virtual BaseKeyType classID() const;

};

template< typename Storage >
class PersistentStorage< Derived::DerivedParams, Storage >
: public PersistentStorage< Base::BaseParams, Storage >
{
    Derived::DerivedParams param_;
    typedef PersistentStorage< Base::BaseParams, Storage > Parent;
public:
    explicit PersistentStorage( Storage & warehouse_ )
        : PersistentStorage< Base::BaseParams, Storage >( warehouse_ )
    {
    }
    Derived::DerivedParams & restore()
    {
        std::cout << "Restoring DerivedParams" << std::endl;
        param_ = Parent::restore();
        param_.word_ = PersistentStorage< std::string, Storage>( warehouse_ ).restore();
        return param_;
    }
    void save( const Derived::DerivedParams & params )
    {
        Parent::save( params );
        PersistentStorage< std::string, Storage >( warehouse_ ).save( params.word_ );
    }
};

#endif
