// main.cpp
// Example and test file to illustrate how to use the factory template.
// Copyright 2001, 2003 Jim Hyslop.
//
// This file may be freely used, modified and distributed, provided that
// the accompanying copyright notice remains intact.
//
// $Id: main.cpp,v 1.16 2003/03/09 16:29:08 Jim Exp $

#include <iostream>
#include <fstream>
#include "base.h"
#include <vector>
#include "factory.h"
#include <boost/shared_ptr.hpp>

#include "base_stream.h"
using namespace std;
using namespace boost;

// In a production environment, there would of course be much stricter
// error checking.
auto_ptr< Base > createNewObject(const Base::BaseKeyType &objectType)
{
    return GenericFactory<Base>::instance().create(objectType);
}


const int extraMenuItemStart=4;
int main()
{
    int menuCount;
    int userChoice;
    const string saveFileName("save.txt");
    BaseFactoryUsingStreams & factory = BaseFactoryUsingStreams::instance();

    // Get a list of the classes that are registered, to add to the menu options
    vector<string> classes ( factory.getRegisteredClasses() );

    vector< shared_ptr< Base > > loadedClasses;
    do {
        int count;
        // Note the lack of error checking - to crash this program, enter a number
        // outside the expected range.
        cout << "\nEnter a choice\n"
            "0: quit\n"
            "1: Load file\n"
            "2: Save file\n"
            "3: Call virtual functions on all objects\n";
        for (menuCount=0; menuCount<classes.size(); menuCount++) {
            cout << menuCount+extraMenuItemStart << 
                ": Create new object of type " << classes[menuCount] << endl;
        }
        menuCount+=extraMenuItemStart;
        cin >> userChoice;
        switch (userChoice) {
        case 1:
            {
                ifstream infile(saveFileName.c_str());
                loadInputFile(infile, loadedClasses);
            }
            break;
        case 2:
            { // block scope for local variable
                ofstream outfile(saveFileName.c_str());
                saveOutputFile(outfile, loadedClasses);
            }
            break;
        case 3:
            for (count=0; count < loadedClasses.size(); ++count) {
                loadedClasses[count]->displayValues();
            }
            break;
        case 0:
            // do nothing
            break;
        default:
            auto_ptr< Base > newBase;
            newBase = createNewObject(classes[userChoice-extraMenuItemStart]);
            loadedClasses.push_back( shared_ptr< Base >( newBase ) );
            break;
        }
    } while (userChoice != 0);

    return 0;
}


