// Copyright 2003 Jim Hyslop.
//
// This file may be freely used, modified and distributed, provided that
// the accompanying copyright notice remains intact.
//
// $Id: persistent_storage_stream_specializations.h,v 1.4 2003/04/02 03:47:53 Jim Exp $
#ifndef PERSISTENT_STORAGE_tEMPLATE_STREAM_SPECIALIZATIONS
#define PERSISTENT_STORAGE_tEMPLATE_STREAM_SPECIALIZATIONS

template<>
class PersistentStorage< int, std::istream >
{
    int obj_;
public:
    explicit PersistentStorage( std::istream & s ) 
    {
        s >> obj_;
        s.get(); // eat delimiter
    }
    int & restore()
    {
        return obj_;
    }
};

template<>
class PersistentStorage< int, std::ostream >
{
    std::ostream & warehouse_;
public:
    explicit PersistentStorage( std::ostream & s ) : warehouse_( s ) {}
    void save( const int & t )
    {
        warehouse_ << t << ' ';
    }
};

template<>
class PersistentStorage< unsigned, std::istream >
{
    unsigned obj_;
public:
    explicit PersistentStorage( std::istream & s ) 
    {
        s >> obj_;
        s.get(); // eat delimiter
    }
    unsigned & restore()
    {
        return obj_;
    }
};

template<>
class PersistentStorage< unsigned, std::ostream >
{
    std::ostream & warehouse_;
public:
    explicit PersistentStorage( std::ostream & s ) : warehouse_( s ) {}
    void save( const unsigned & t )
    {
        warehouse_ << t << ' ';
    }
};

template< >
class PersistentStorage< std::string, std::istream >
{
    std::string string_;
public:
    explicit PersistentStorage( std::istream & s )
    {
        std::string::size_type numElems = 
            PersistentStorage< std::string::size_type, std::istream>( s ).restore();
        std::vector<char> incoming;
        incoming.resize( numElems+1 );
        s.read( &incoming[0], numElems );
        incoming[numElems]='\0';
        string_ = &incoming[0];
    }
    const std::string & restore()
    {
        return string_;
    }
};

template< >
class PersistentStorage< std::string, std::ostream >
{
    std::ostream & warehouse_;
public:
    explicit PersistentStorage( std::ostream & s ) : warehouse_( s ) {}
    void save( const std::string & theString )
    {
        PersistentStorage< std::string::size_type, std::ostream>( warehouse_ ).save( theString.size() );
        warehouse_ << theString;
    }
};

#endif

